/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controllers.relation_calc

import com.persagy.database.SFilter
import com.persagy.server.datacenter.models.entities.graphtype.block.MepSourceTypeEntity
import com.persagy.server.services.objects.EquipmentService
import com.persagy.server.services.relation_calc.MepSystemCalcService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*


/**
 * 机电管网系统关系计算
 *
 * @author jxing
 */
@Tag(name = "014、关系计算-管网计算")
@RestController
@RequestMapping("/mep-system-calc")
class MepSystemCalcController {

    @Autowired
    lateinit var mepSystemCalc: MepSystemCalcService

    /**
    * 添加源设备
    */
    @Operation(summary = "添加源设备", description = "")
    @PostMapping(value = ["/source/add"])
    fun query(@RequestBody entity: SCreateRequest<MepSourceTypeEntity>): SCreateResponse<MepSourceTypeEntity> {
        val projectId = SPageContext.getHeader("projectId").toString()
        var calcName : String? = null
        var sourceType : String? = null
        entity.content.forEach {
            if(it.projectId.isNullOrBlank())
                it.projectId = projectId
            calcName = it.calcName
            sourceType = it.sourceType
        }
        // 删除以前卡片的源设备记录
        mepSystemCalc.mepSourceTypeService.delete(SFilter.and(SFilter.eq("projectId", projectId), SFilter.eq("calcName", calcName!!)))
        if(sourceType == null){
            return SCreateResponse<MepSourceTypeEntity>(SResponseType.success,"")
        }
        return mepSystemCalc.mepSourceTypeService.createList(entity)
    } // Fun

    /**
     * 添加关系
     */
    @Operation(summary = "查看源设备", description = "")
    @GetMapping(value = ["/source/query"])
    fun addRels(@RequestParam calcName: String): SQueryResponse<MepSourceTypeEntity> {
        val projectId = SPageContext.getHeader("projectId").toString()
        val mems = mepSystemCalc.mepSourceTypeService.select(SFilter.eq("projectId", projectId), SFilter.eq("calcName", calcName!!)).order("calcName").exec()
        for (mem in mems) {
            val eCounts = EquipmentService.select(SFilter.eq("projectId", projectId),SFilter.eq("classCode", mem.sourceType!!)).count
            if(eCounts != null){
                mem.sourceCount = eCounts
            }else {
                mem.sourceCount = 0
            }
        }
        val response = SQueryResponse<MepSourceTypeEntity>()
        response.content = mems
        return  response
    } // Fun
}