/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2021.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.services.journal

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.syn.jms.DataCenterSync
import com.persagy.server.syn.models.api.obj.ProjectApi
import com.persagy.server.syn.models.tools.ProjectUtil
import com.persagy.server.syn.services.BaseDataService
import com.persagy.server.syn.services.dict.DClassDefService
import com.persagy.server.syn.services.mappers.TypeMapper
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SQueryResponse
import com.persagy.service.utils.SSpringContextUtil

/**
 * 日志 服务
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2021/2/25 10:33
 */
object SynchronousMessageService : SObjectService<SynchronousMessage>(SMybatisDao(SynchronousMessage::class.java)) {



    /** 类型统计 */
    private val typeMapper by lazy {
        SSpringContextUtil.getBean(TypeMapper::class.java) as TypeMapper
    }

    /** 数据中心处理数据同步到数据中台 */
    private val dataCenterSync by lazy {
        SSpringContextUtil.getBean(DataCenterSync::class.java) as DataCenterSync
    }


    /**
     * 查询未同步的对象分类
     *
     * @param objType   对象类型
     */
    fun typeQuery(objType: String,projectId: String): SQueryResponse<DefClass> {
        try {
            val sQueryResponse = SQueryResponse<DefClass>(SResponseType.success)
            val typeCountList = typeMapper.typeCount(projectId, objType)
            if (!typeCountList.isNullOrEmpty()) {
                val listCode = ArrayList<String>()
                for (typeCount in typeCountList){
                    listCode.add(typeCount.code!!)
                }
                val dClassDefList = DClassDefService.select( SFilter.inList("code", listCode)).exec()
                sQueryResponse.content = dClassDefList
                sQueryResponse.total = dClassDefList.size.toLong()
            }else {
                sQueryResponse.content = ArrayList()
            }
            return sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return  SQueryResponse( SResponseType.failure,e.message.toString())
        }

    }

    /**
     *  重新同步数据
     *
     *  @param request  日志对象列表
     */
    fun synData(request: SUpdateRequest<SynchronousMessage>): SBaseResponse {
        if (!request.content.isNullOrEmpty()) {
            dataCenterSync.dataSync(request.content!!)
        }
        return SBaseResponse(SResponseType.success)
    }

    /**
     * 测试服务器是否在线
     *
     * @param groupCode  集团编码
     */
    fun isOnline(groupCode: String,projectId: String): SBaseResponse{
        try {
            val sBaseResponse = SBaseResponse()
            val postObject = ProjectApi.projectQueryTwo(groupCode,projectId)
            if (postObject!=null&&postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                if (postObject.data!!.size > 0) {
                    for (projectQuery in postObject.data!!) {
                        val projectDataConversion = ProjectUtil.projectDataConversions(groupCode, projectQuery)
                        BaseDataService.projectService.replace(projectDataConversion)
                    }
                }
                sBaseResponse.result = SResponseType.success
                sBaseResponse.message = "服务正常"
            } else {
                sBaseResponse.result = SResponseType.failure
                sBaseResponse.message = "服务器异常"
            }
            return sBaseResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,"服务器异常")
        }
    }



}