/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.models

import com.persagy.server.datacenter.models.entities.assistant.BIMLocation
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 资产信息实体类
 *
 * @author  张维新
 */
@Schema(description = "资产信息实体类")
@Table(name = "v_property_supplement")
open class PropertySupplement : BaseInfo(),Comparator<PropertySupplement> {

    /** 资产id */
    @Schema(description = "资产id", required = true)
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 资产名称 */
    @Schema(description = "资产名称", required = true)
    @Column(name = "name")
    var name: String? = null

    /** 资产本地编码 */
    @Schema(description = "资产本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 资产本地名称 */
    @Schema(description = "资产本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 资产二维码照片 */
    @Schema(description = "资产二维码照片")
    @Column(name = "qr_code")
    var qrCode: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    var cadId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null


    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标json")
    var locationJson: BIMLocation? = null
        get() {
            if (!bimLocation.isNullOrEmpty()){

                val split = bimLocation!!.split(",")
                var bimLocation = BIMLocation()
                if (split.size == 3){
                    bimLocation!!.x = split[0].toDouble()
                    bimLocation!!.y = split[1].toDouble()
                    bimLocation!!.z = split[2].toDouble()
                }else if (split.size == 2){

                    bimLocation!!.x = split[0].toDouble()
                    bimLocation!!.y = split[1].toDouble()
                    bimLocation!!.z = 0.0
                }
                return bimLocation
            }
            return null
        }

    /** 项目id */
    @Schema(description = "项目id", required = true)
    @Column(name = "project_id")
    var projectId: String? = null

    /** 资产所在建筑id */
    @Schema(description = "资产所在建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** 资产所在楼层id */
    @Schema(description = "资产所在楼层id")
    @Column(name = "floor_id")
    var floorId: String? = null

    /** 信息点 */
    @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String,Any?>? = null


    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    var modelId: String? = null

    /** 类型 */
    @Schema(description = "类型")
    @Column(name = "class_code")
    var classCode: String? = null

    /** 类型名 */
    @Schema(description = "类型名")
    @Column(name = "code_name")
    var codeName: String? = null


    /** 距离*/
    @Schema(description = "距离")
    var distance: Double? = null

    override fun compare(o1: PropertySupplement?, o2: PropertySupplement?): Int {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
        if (o1!!.distance!=null&&o2!!.distance!=null&&o1!!.distance!!>o2.distance!!){
            return 1
        }else{
            -1
        }
    }

    /** 楼层名称 */
    @Schema(description = "楼层名称")
    @Column(name = "floor_name")
    var floorName: String? = null

    /** 建筑名称 */
    @Schema(description = "建筑名称")
    @Column(name = "building_name")
    var buildingName: String? = null

    /** 现场发现人 */
    @Schema(description = "现场发现人")
    @Column(name = "find_people")
    var findPeople: String? = null

}